import fetch from 'cross-fetch';
import { sonarQubeApiRef, useProjectInfo, SONARQUBE_PROJECT_KEY_ANNOTATION as SONARQUBE_PROJECT_KEY_ANNOTATION$1, isSonarQubeAvailable as isSonarQubeAvailable$1 } from '@backstage/plugin-sonarqube-react';
import { useEntity, MissingAnnotationEmptyState } from '@backstage/plugin-catalog-react';
import Chip from '@material-ui/core/Chip';
import Grid from '@material-ui/core/Grid';
import { makeStyles, useTheme, lighten } from '@material-ui/core/styles';
import BugReport from '@material-ui/icons/BugReport';
import Lock from '@material-ui/icons/Lock';
import LockOpen from '@material-ui/icons/LockOpen';
import Security from '@material-ui/icons/Security';
import SentimentVeryDissatisfied from '@material-ui/icons/SentimentVeryDissatisfied';
import SentimentVerySatisfied from '@material-ui/icons/SentimentVerySatisfied';
import React, { useMemo } from 'react';
import useAsync from 'react-use/esm/useAsync';
import { Circle } from 'rc-progress';
import Avatar from '@material-ui/core/Avatar';
import { Link, InfoCard, Progress, EmptyState } from '@backstage/core-components';
import Typography from '@material-ui/core/Typography';
import { useApi, createPlugin, createApiFactory, discoveryApiRef, identityApiRef, createComponentExtension } from '@backstage/core-plugin-api';
import { DateTime } from 'luxon';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class SonarQubeClient {
  constructor(options) {
    __publicField(this, "discoveryApi");
    __publicField(this, "identityApi");
    this.discoveryApi = options.discoveryApi;
    this.identityApi = options.identityApi;
  }
  async callApi(path, query) {
    const { token: idToken } = await this.identityApi.getCredentials();
    const apiUrl = `${await this.discoveryApi.getBaseUrl("sonarqube")}`;
    const response = await fetch(
      `${apiUrl}/${path}?${new URLSearchParams(query).toString()}`,
      {
        headers: {
          "Content-Type": "application/json",
          ...idToken && { Authorization: `Bearer ${idToken}` }
        }
      }
    );
    if (response.status === 200) {
      return await response.json();
    }
    return void 0;
  }
  async getFindingSummary({
    componentKey,
    projectInstance
  } = {}) {
    if (!componentKey) {
      return void 0;
    }
    const instanceKey = projectInstance || "";
    const metrics = {
      alert_status: void 0,
      bugs: void 0,
      reliability_rating: void 0,
      vulnerabilities: void 0,
      security_rating: void 0,
      security_hotspots_reviewed: void 0,
      security_review_rating: void 0,
      code_smells: void 0,
      sqale_rating: void 0,
      coverage: void 0,
      duplicated_lines_density: void 0
    };
    const baseUrlWrapper = await this.callApi(
      "instanceUrl",
      {
        instanceKey
      }
    );
    let baseUrl = baseUrlWrapper == null ? void 0 : baseUrlWrapper.instanceUrl;
    if (!baseUrl) {
      return void 0;
    }
    if (!baseUrl.endsWith("/")) {
      baseUrl += "/";
    }
    const findings = await this.callApi("findings", {
      componentKey,
      instanceKey
    });
    if (!findings) {
      return void 0;
    }
    findings.measures.forEach((m) => {
      metrics[m.metric] = m.value;
    });
    return {
      lastAnalysis: findings.analysisDate,
      metrics,
      projectUrl: `${baseUrl}dashboard?id=${encodeURIComponent(componentKey)}`,
      getIssuesUrl: (identifier) => `${baseUrl}project/issues?id=${encodeURIComponent(
        componentKey
      )}&types=${identifier.toLocaleUpperCase("en-US")}&resolved=false`,
      getComponentMeasuresUrl: (identifier) => `${baseUrl}component_measures?id=${encodeURIComponent(
        componentKey
      )}&metric=${identifier.toLocaleLowerCase(
        "en-US"
      )}&resolved=false&view=list`,
      getSecurityHotspotsUrl: () => `${baseUrl}${baseUrl === "https://sonarcloud.io/" ? "project/" : ""}security_hotspots?id=${encodeURIComponent(componentKey)}`
    };
  }
}

const useStyles$4 = makeStyles((theme) => ({
  root: {
    height: theme.spacing(3),
    width: theme.spacing(3)
  }
}));
const Percentage = ({ value }) => {
  const classes = useStyles$4();
  const theme = useTheme();
  return /* @__PURE__ */ React.createElement(
    Circle,
    {
      strokeLinecap: "butt",
      percent: +(value || 0),
      strokeWidth: 16,
      strokeColor: theme.palette.status.ok,
      trailColor: theme.palette.status.error,
      trailWidth: 16,
      className: classes.root
    }
  );
};

const useStyles$3 = makeStyles((theme) => {
  const commonCardRating = {
    height: theme.spacing(3),
    width: theme.spacing(3),
    color: theme.palette.common.white
  };
  return {
    ratingDefault: {
      ...commonCardRating,
      background: theme.palette.status.aborted
    },
    ratingA: {
      ...commonCardRating,
      background: theme.palette.status.ok
    },
    ratingB: {
      ...commonCardRating,
      background: lighten(theme.palette.status.ok, 0.5)
    },
    ratingC: {
      ...commonCardRating,
      background: theme.palette.status.pending
    },
    ratingD: {
      ...commonCardRating,
      background: theme.palette.status.warning
    },
    ratingE: {
      ...commonCardRating,
      background: theme.palette.error.main
    }
  };
});
const Rating = ({
  rating,
  hideValue
}) => {
  const classes = useStyles$3();
  const ratingProp = useMemo(() => {
    switch (rating) {
      case "1.0":
        return {
          name: "A",
          className: classes.ratingA
        };
      case "2.0":
        return {
          name: "B",
          className: classes.ratingB
        };
      case "3.0":
        return {
          name: "C",
          className: classes.ratingC
        };
      case "4.0":
        return {
          name: "D",
          className: classes.ratingD
        };
      case "5.0":
        return {
          name: "E",
          className: classes.ratingE
        };
      default:
        return {
          name: "",
          className: classes.ratingDefault
        };
    }
  }, [classes, rating]);
  return /* @__PURE__ */ React.createElement(Avatar, { className: ratingProp.className }, !hideValue && ratingProp.name);
};

const useStyles$2 = makeStyles((theme) => {
  return {
    root: {
      margin: theme.spacing(1, 0),
      minWidth: "140px"
    },
    upper: {
      display: "flex",
      justifyContent: "center",
      alignItems: "center"
    },
    cardTitle: {
      textAlign: "center"
    },
    wrapIcon: {
      display: "inline-flex",
      verticalAlign: "baseline"
    },
    left: {
      display: "flex"
    },
    right: {
      display: "flex",
      marginLeft: theme.spacing(0.5)
    }
  };
});
const RatingCard = ({
  leftSlot,
  rightSlot,
  title,
  titleIcon,
  link
}) => {
  const classes = useStyles$2();
  return /* @__PURE__ */ React.createElement(Link, { to: link, color: "inherit", underline: "none" }, /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.root }, /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.upper }, /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.left }, leftSlot), /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.right }, rightSlot)), /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.cardTitle }, /* @__PURE__ */ React.createElement(Typography, { variant: "body1", className: classes.wrapIcon }, titleIcon, " ", title))));
};

const useStyles$1 = makeStyles((theme) => {
  return {
    value: {
      fontSize: "1.5rem",
      fontWeight: theme.typography.fontWeightMedium
    }
  };
});
const Value = (props) => {
  const classes = useStyles$1();
  return /* @__PURE__ */ React.createElement(Typography, { component: "span", className: classes.value }, props.value);
};

const useStyles = makeStyles((theme) => ({
  badgeLabel: {
    color: theme.palette.common.white
  },
  badgeError: {
    margin: 0,
    backgroundColor: theme.palette.error.main
  },
  badgeSuccess: {
    margin: 0,
    backgroundColor: theme.palette.success.main
  },
  badgeUnknown: {
    backgroundColor: theme.palette.grey[500]
  },
  header: {
    padding: theme.spacing(2, 2, 2, 2.5)
  },
  action: {
    margin: 0
  },
  lastAnalyzed: {
    color: theme.palette.text.secondary
  }
}));
const defaultDuplicationRatings = [
  { greaterThan: 0, rating: "1.0" },
  { greaterThan: 3, rating: "2.0" },
  { greaterThan: 5, rating: "3.0" },
  { greaterThan: 10, rating: "4.0" },
  { greaterThan: 20, rating: "5.0" }
];
const SonarQubeCard = (props) => {
  const {
    variant = "gridItem",
    duplicationRatings = defaultDuplicationRatings
  } = props;
  const { entity } = useEntity();
  const sonarQubeApi = useApi(sonarQubeApiRef);
  const { projectKey: projectTitle, projectInstance } = useProjectInfo(entity);
  const { value, loading } = useAsync(
    async () => sonarQubeApi.getFindingSummary({
      componentKey: projectTitle,
      projectInstance
    }),
    [sonarQubeApi, projectTitle]
  );
  const deepLink = !loading && value ? {
    title: "View more",
    link: value.projectUrl
  } : void 0;
  const classes = useStyles();
  let gateLabel = "Not computed";
  let gateColor = classes.badgeUnknown;
  if (value == null ? void 0 : value.metrics.alert_status) {
    const gatePassed = value.metrics.alert_status === "OK";
    gateLabel = gatePassed ? "Gate passed" : "Gate failed";
    gateColor = gatePassed ? classes.badgeSuccess : classes.badgeError;
  }
  const qualityBadge = !loading && value && /* @__PURE__ */ React.createElement(
    Chip,
    {
      label: gateLabel,
      classes: { root: gateColor, label: classes.badgeLabel }
    }
  );
  const duplicationRating = useMemo(() => {
    if (loading || !value || !value.metrics.duplicated_lines_density) {
      return "";
    }
    let rating = "";
    for (const r of duplicationRatings) {
      if (+value.metrics.duplicated_lines_density >= r.greaterThan) {
        rating = r.rating;
      }
    }
    return rating;
  }, [loading, value, duplicationRatings]);
  return /* @__PURE__ */ React.createElement(
    InfoCard,
    {
      title: "Code Quality",
      deepLink,
      variant,
      headerProps: {
        action: qualityBadge,
        classes: {
          root: classes.header,
          action: classes.action
        }
      }
    },
    loading && /* @__PURE__ */ React.createElement(Progress, null),
    !loading && !projectTitle && /* @__PURE__ */ React.createElement(
      MissingAnnotationEmptyState,
      {
        annotation: SONARQUBE_PROJECT_KEY_ANNOTATION$1
      }
    ),
    !loading && projectTitle && !value && /* @__PURE__ */ React.createElement(
      EmptyState,
      {
        missing: "info",
        title: "No information to display",
        description: `There is no SonarQube project with key '${projectTitle}'.`
      }
    ),
    !loading && value && /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(
      Grid,
      {
        item: true,
        container: true,
        direction: "column",
        justifyContent: "space-between",
        alignItems: "center",
        style: { height: "100%" },
        spacing: 0
      },
      /* @__PURE__ */ React.createElement(Grid, { item: true, container: true, justifyContent: "space-around" }, /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          titleIcon: /* @__PURE__ */ React.createElement(BugReport, null),
          title: "Bugs",
          link: value.getIssuesUrl("BUG"),
          leftSlot: /* @__PURE__ */ React.createElement(Value, { value: value.metrics.bugs }),
          rightSlot: /* @__PURE__ */ React.createElement(Rating, { rating: value.metrics.reliability_rating })
        }
      ), /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          titleIcon: value.metrics.vulnerabilities === "0" ? /* @__PURE__ */ React.createElement(Lock, null) : /* @__PURE__ */ React.createElement(LockOpen, null),
          title: "Vulnerabilities",
          link: value.getIssuesUrl("VULNERABILITY"),
          leftSlot: /* @__PURE__ */ React.createElement(Value, { value: value.metrics.vulnerabilities }),
          rightSlot: /* @__PURE__ */ React.createElement(Rating, { rating: value.metrics.security_rating })
        }
      ), /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          titleIcon: value.metrics.code_smells === "0" ? /* @__PURE__ */ React.createElement(SentimentVerySatisfied, null) : /* @__PURE__ */ React.createElement(SentimentVeryDissatisfied, null),
          title: "Code Smells",
          link: value.getIssuesUrl("CODE_SMELL"),
          leftSlot: /* @__PURE__ */ React.createElement(Value, { value: value.metrics.code_smells }),
          rightSlot: /* @__PURE__ */ React.createElement(Rating, { rating: value.metrics.sqale_rating })
        }
      ), value.metrics.security_review_rating && /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          titleIcon: /* @__PURE__ */ React.createElement(Security, null),
          title: "Hotspots Reviewed",
          link: value.getSecurityHotspotsUrl(),
          leftSlot: /* @__PURE__ */ React.createElement(
            Value,
            {
              value: value.metrics.security_hotspots_reviewed ? `${value.metrics.security_hotspots_reviewed}%` : "\u2014"
            }
          ),
          rightSlot: /* @__PURE__ */ React.createElement(Rating, { rating: value.metrics.security_review_rating })
        }
      ), /* @__PURE__ */ React.createElement("div", { style: { width: "100%" } }), /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          link: value.getComponentMeasuresUrl("COVERAGE"),
          title: "Coverage",
          leftSlot: /* @__PURE__ */ React.createElement(Percentage, { value: value.metrics.coverage }),
          rightSlot: /* @__PURE__ */ React.createElement(
            Value,
            {
              value: value.metrics.coverage !== void 0 ? `${value.metrics.coverage}%` : "\u2014"
            }
          )
        }
      ), /* @__PURE__ */ React.createElement(
        RatingCard,
        {
          title: "Duplications",
          link: value.getComponentMeasuresUrl("DUPLICATED_LINES_DENSITY"),
          leftSlot: /* @__PURE__ */ React.createElement(Rating, { rating: duplicationRating, hideValue: true }),
          rightSlot: /* @__PURE__ */ React.createElement(Value, { value: `${value.metrics.duplicated_lines_density}%` })
        }
      )),
      /* @__PURE__ */ React.createElement(Grid, { item: true, className: classes.lastAnalyzed }, "Last analyzed on", " ", DateTime.fromISO(value.lastAnalysis).toLocaleString(
        DateTime.DATETIME_MED
      ))
    ))
  );
};

const isSonarQubeAvailable = isSonarQubeAvailable$1;
const SONARQUBE_PROJECT_KEY_ANNOTATION = SONARQUBE_PROJECT_KEY_ANNOTATION$1;

const sonarQubePlugin = createPlugin({
  id: "sonarqube",
  apis: [
    createApiFactory({
      api: sonarQubeApiRef,
      deps: {
        discoveryApi: discoveryApiRef,
        identityApi: identityApiRef
      },
      factory: ({ discoveryApi, identityApi }) => new SonarQubeClient({
        discoveryApi,
        identityApi
      })
    })
  ]
});
const EntitySonarQubeCard = sonarQubePlugin.provide(
  createComponentExtension({
    name: "EntitySonarQubeCard",
    component: {
      lazy: () => import('./esm/index-DmJcy74T.esm.js').then((m) => m.SonarQubeCard)
    }
  })
);
const EntitySonarQubeContentPage = sonarQubePlugin.provide(
  createComponentExtension({
    name: "EntitySonarQubeContentPage",
    component: {
      lazy: () => import('./esm/index-BGkJlRMA.esm.js').then(
        (m) => m.SonarQubeContentPage
      )
    }
  })
);

export { EntitySonarQubeCard, EntitySonarQubeContentPage, SONARQUBE_PROJECT_KEY_ANNOTATION, SonarQubeCard, SonarQubeClient, isSonarQubeAvailable, sonarQubePlugin };
//# sourceMappingURL=index.esm.js.map
